﻿using Microsoft.Xna.Framework;

namespace Utilities
{
	/// <summary>
	///     This is a helper class that represents a millisecond-timer.
	///     You may change its value (timer) or interval even during run-time.
	///     <para>
	///         <code>
	///           This code is published using the MIT license:
	///          
	///           Copyright (c) 2012 Unterrainer Informatik OG
	///           Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
	///           documentation files (the "Software"), to deal in the Software without restriction, including without limitation
	///           the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
	///           and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
	///           The above copyright notice and this permission notice shall be included in all copies or substantial portions
	///           of the Software.
	///           THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
	///           TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	///           THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
	///           CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	///           DEALINGS IN THE SOFTWARE.
	///  		</code>
	///         In plain english:
	///         You may use it, free of charge, in all possible ways, even in commercial projects.
	///         The use of this software is your responsibility, as well as the resulting consequences.
	///         It is not copy-left and I will not even sue you, if you're removing this notice and tell everybody that this
	///         code was your idea. Promised.
	///         If you feel like it you may credit me or our site http://www.unterrainer.info, or both, but you don't have to.
	///     </para>
	/// </summary>
	public class IntervalTimer
	{
		private float _timerInMilliseconds;
		private float _intervalInMilliseconds;

		/// <summary>
		///     Initializes a new instance of the <see cref="IntervalTimer" /> class.
		/// </summary>
		/// <param name="intervalInMilliseconds">The interval in milliseconds.</param>
		public IntervalTimer(float intervalInMilliseconds)
		{
			_intervalInMilliseconds = intervalInMilliseconds;
			_timerInMilliseconds = 0;
		}

		/// <summary>
		///     Gets or sets the timer in milliseconds.
		/// </summary>
		/// <value>The timer in milliseconds.</value>
		public float TimerInMilliseconds
		{
			get { return _timerInMilliseconds; }
			set { _timerInMilliseconds = value; }
		}

		/// <summary>
		///     Gets or sets the interval in milliseconds.
		/// </summary>
		/// <value>The interval in milliseconds.</value>
		public float IntervalInMilliseconds
		{
			get { return _intervalInMilliseconds; }
			set { _intervalInMilliseconds = value; }
		}

		/// <summary>
		///     Gets the time left to go until reset.
		/// </summary>
		/// <value>The time left to go until reset.</value>
		public float TimeLeftToGoUntilReset
		{
			get { return _intervalInMilliseconds - _timerInMilliseconds; }
		}

		/// <summary>
		///     Gets the percentage of the interval that passed since the last
		///     reset. This method returns a  value between 0 and 1, where 0 means
		///     that the interval has just started and 1 means that it is about to
		///     reset.
		/// </summary>
		/// <value>The time left to go until reset.</value>
		public float PercentageElapsedSinceLastReset
		{
			get { return _timerInMilliseconds/_intervalInMilliseconds; }
		}

		/// <summary>
		///     Resets the timer.
		/// </summary>
		public void ResetTimer()
		{
			_timerInMilliseconds = 0f;
		}

		/// <summary>
		///     Sets the timer so that it fires when calling update the next time.
		/// </summary>
		public void SetTimer()
		{
			_timerInMilliseconds = _intervalInMilliseconds;
		}

		/// <summary>
		///     Updates the timer and returns <see langword="true" /> if it has
		///     fired.
		/// </summary>
		/// <param name="gameTime">The game time.</param>
		/// <param name="isReassignOverlap">
		///     If set to <c>true</c> it reassigns the overlap (the rest of the passed time) to the timer (modulo timer-length of
		///     course),
		///     in order to loose no time.
		/// </param>
		/// <returns>
		///     <see langword="true" />, if it has fired; Otherwise
		///     <see langword="false" />.
		/// </returns>
		public bool Update(GameTime gameTime, bool isReassignOverlap = false)
		{
			return Update((float) gameTime.ElapsedGameTime.TotalMilliseconds, isReassignOverlap);
		}

		/// <summary>
		///     Updates the timer and returns <see langword="true" /> if it has
		///     fired.
		/// </summary>
		/// <param name="elapsedTimeInMilliseconds">The elapsed time in milliseconds.</param>
		/// <param name="isReassignOverlap">
		///     If set to <c>true</c> it reassigns the overlap (the rest of the passed time) to the timer (modulo timer-length of
		///     course),
		///     in order to loose no time.
		/// </param>
		/// <returns>
		///     <see langword="true" />, if it has fired; Otherwise
		///     <see langword="false" />.
		/// </returns>
		public bool Update(float elapsedTimeInMilliseconds = 0f, bool isReassignOverlap = false)
		{
			_timerInMilliseconds += elapsedTimeInMilliseconds;
			if (_timerInMilliseconds >= _intervalInMilliseconds)
			{
				if (isReassignOverlap)
				{
					float rest = _timerInMilliseconds - _intervalInMilliseconds;
					rest = rest%_intervalInMilliseconds;
					_timerInMilliseconds = rest;
				}
				else
				{
					_timerInMilliseconds = 0f;
				}
				return true;
			}
			return false;
		}
	}
}